#ifndef NETGEN_VERSION_HPP_INCLUDED
#define NETGEN_VERSION_HPP_INCLUDED
#define NETGEN_VERSION "6.2.2404-80-g7656211b"
#define NETGEN_VERSION_MAJOR 6
#define NETGEN_VERSION_MINOR 2
#define NETGEN_VERSION_PATCH 2404
#define NETGEN_VERSION_TWEAK 80
#define NETGEN_VERSION_HASH "g7656211b"
#endif // NETGEN_VERSION_HPP_INCLUDED
